import 'dart:async';
import 'package:flutter/material.dart';
import 'package:camera/camera.dart';
import 'dart:io';

import 'preview.dart';
import 'listCameraFlashMode.dart';

CameraController? _cameraController;

class OpenCamera extends StatefulWidget {
final int totalQuestion;
final bool switchCamera;
  OpenCamera(
     {
    Key? key,this.totalQuestion = 1,this.switchCamera = false
  }) : super(key: key);

  @override
  _OpenCameraState createState() => _OpenCameraState();
}

class _OpenCameraState extends State<OpenCamera> {
  List<XFile> listImageFile = [];
  late File file;
  late int cameraType;
  late int flashMode;
  List<CameraDescription>? cameras;
  ListCameraFlashMode _listCameraFlashMode = ListCameraFlashMode();

  @override
  void initState() {

    super.initState();
    cameraType = 0;
    flashMode = 0;
    getCamera();

  }
  void getCamera()async{
    cameras = await availableCameras();
    setState(() {

    });
  }
  @override
  Widget build(BuildContext context) {
    if(cameras == null){
      return Container();
    }
    return WillPopScope(
      onWillPop: () async {
        Navigator.pop(context, listImageFile);
        return false;
      },
      child: Scaffold(
        body: Column(
          children: [
            Expanded(
              child: Stack(
                children: [
                  CameraApp(cameras![0], UniqueKey),
                  Positioned(bottom:0,child: buildMiniPreview()),
                ],
              ),
            ),
            Container(
              height: 100,
              color: Colors.white60,
              child: Builder(builder: (context){
                if(widget.switchCamera){
                  return Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    children: [
                      DoneButton(context),
                      Column(
                        children: [
                          cameraControl(),
                          switchCamera(),
                        ],
                      ),
                      flashButtonSwitch(),
                    ],
                  );
                }
                return Row(
                  mainAxisAlignment: MainAxisAlignment.spaceAround,
                  children: [
                    DoneButton(context),
                    cameraControl(),
                    flashButtonSwitch(),
                  ],
                );
              } ,
              ),
            ),

          ],
        ),
      ),
    );
  }

  GestureDetector DoneButton(BuildContext context) {
    return GestureDetector(
                                    onTap: (){   Navigator.pop(context, listImageFile);},
                                    child: Icon(Icons.done,color: Colors.black,));
  }

  GestureDetector flashButtonSwitch() {
    return GestureDetector(
                          onTap: () {
                            if (flashMode ==
                                _listCameraFlashMode.listCameraFlashMode.length - 1) {
                              flashMode = 0;
                            } else {
                              flashMode = flashMode + 1;
                            }
                            _cameraController!.setFlashMode(_listCameraFlashMode
                                .listCameraFlashMode[flashMode].flashMode);

                            setState(() {});
                          },
                          child: Icon(
                              _listCameraFlashMode.listCameraFlashMode[flashMode].iconData,
                              color: Colors.black));
  }

  GestureDetector switchCamera() {
    return GestureDetector(
        onTap: () {
          cameraType = cameraType == 0 ? 1 : 0;
          _cameraController = CameraController(
              cameras![cameraType], ResolutionPreset.max);
          _cameraController!.initialize().then((_) {
            setState(() {});
          });
        },
        child: Icon(Icons.swap_horiz,color: Colors.black,));
  }

  GestureDetector cameraControl() {

    return GestureDetector(
        onTap: () {
          onTakePictureButtonPressed();
        },
        child: Icon(
          Icons.camera,
          color: Colors.black,
          size: 60,
        ));
  }

  Container buildMiniPreview() {

    return Container(
        height: 70,
        width: MediaQuery.of(context).size.width,
        child: listImageFile.length != 0
            ? ListView.builder(
          scrollDirection: Axis.horizontal,
          itemCount: listImageFile.length,itemBuilder: (context,index){
          return Container(
            width: 70,
            child: Stack(children: [
              Align(
                  alignment: Alignment.center,
                  child: GestureDetector(
                      onTap: () async{
                        await Navigator.push(
                          context,
                          MaterialPageRoute(
                              builder: (context) => Preview(
                                  listImageFile: listImageFile,
                                  indexOfImage: index,
                                  key: UniqueKey())),
                        );
                        setState(() {

                        });
                        //Navigator.push(context, Preview(key: UniqueKey));
                      },
                      child: Image(
                          image: FileImage(File(listImageFile[
                          listImageFile.length - index - 1]
                              .path))))),
              Positioned(
                right: 5,
                top: 0,
                child: GestureDetector(
                  onTap: () {
                    listImageFile
                        .removeAt(listImageFile.length - index - 1);
                    setState(() {});
                  },
                  child: Icon(
                    Icons.delete,
                    color: Colors.white70,
                    size: 15,
                  ),
                ),
              ),

            ]),
          );
        },)
            : null);
  }

  void onTakePictureButtonPressed() async {
    Future<XFile?> takePicture() async {
      final CameraController? cameraController = _cameraController;
      if (cameraController == null || !cameraController.value.isInitialized) {
        return null;
      }

      if (cameraController.value.isTakingPicture) {
        // A capture is already pending, do nothing.
        return null;
      }

      try {
        XFile file = await cameraController.takePicture();

        return file;
      } on CameraException catch (e) {
        return null;
      }
    }

    takePicture().then((XFile? file) {
      if (file != null) {
        if(widget.totalQuestion ==listImageFile.length){
          listImageFile.removeAt(0);
        }
        listImageFile.add(file);
      }
      setState(() {});
    });
  }
}

class CameraApp extends StatefulWidget {
  CameraDescription cameras;

  CameraApp(this.cameras, key);

  @override
  _CameraAppState createState() => _CameraAppState();
}

class _CameraAppState extends State<CameraApp> {
  @override
  void initState() {
    super.initState();
    _cameraController = CameraController(widget.cameras, ResolutionPreset.max);
    _cameraController!.initialize().then((_) {
      if (!mounted) {
        return;
      }
      setState(() {});
    });
  }

  @override
  void dispose() {
    _cameraController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (!_cameraController!.value.isInitialized) {
      return Container();
    }
    return MaterialApp(
      home: CameraPreview(_cameraController!),
    );
  }
}
